/***************************************************************************
*	MBK : extern variables						   *
*									   *
*	Version : 1.0							   *
*       Date	: 11/06/90						   *
***************************************************************************/

extern phfig_list *xvisufig;
extern LAMBDA xabfig, yabfig;

extern LAMBDA xvisu, yvisu, dxvisu, dyvisu;
extern LAMBDA xscreen, yscreen, dxscreen, dyscreen;

#ifdef TRACE
extern FILE *file_trace;
#endif

extern FILE *xinspect;

extern X_RECT *ptinsref;

extern struct layer tablayer[];
extern char *tabcont[];
extern char *tabtransf[];

extern PIXEL dxWin, dyWin;
extern float scale;

extern X_RECTPEEK *peek_db[];
extern X_RECT *visu_db[];

extern struct zoomBack zoomTrack;

extern X_STRING *head_strlist;

extern char run_mode;

extern long nbins, nbseg, nbvia, nbinscon, nbfigcon, nbref, nbtrans;

/***************************************************************************
*	Functions prototypes						   *
***************************************************************************/

/* g_cache.c functions */
extern void insert_visu ();
extern void del_cache ();
extern X_RECT *fill_trans ();
extern X_RECT *fill_seg ();
extern X_RECT *fill_via ();
extern X_RECT *fill_inscon ();
extern X_RECT *fill_ins ();
extern X_RECT *fill_figcon ();
extern X_RECT *fill_ref ();
extern void fill_descr ();
extern void fill_cache ();
extern void update_AB ();
extern void init_fig ();
extern void load_fig ();

/* g_comp.c functions */
extern int gen_comp ();
extern int gen_exec ();
extern int gen_shell ();

/* g_debug.c functions */
extern void print_list ();
extern void print_str ();
extern void print_all ();

/* g_move.c functions */
extern void pan ();
extern void scroll ();
extern int zoom ();
extern int loom ();
extern void fit ();
extern void resize ();
extern int zoom_mouse ();
extern void loom_mouse ();
extern void xy_move ();

/* g_peek.c functions */
extern void peek_via ();
extern void peek_trans ();
extern void peek_seg ();
extern void peek_con ();
extern int peek ();
extern void unpeek ();

/* g_string.c functions */
extern void string_coord ();

/* g_user.c functions */
extern void g_error ();
extern void update_BB ();
extern int appart_win ();
extern void troncate ();
extern void make_sym ();
extern void seg_coord ();
extern void trans_coord ();
extern void via_coord ();
extern void con_coord ();
extern unsigned char get_layert ();
extern void new_insref ();
